@@ -1,6 +1,13 @@
 import sys
 import traceback
 
+from scalene import scalene_profiler
+
+def should_trace(s) -> bool:
+    if scalene_profiler.Scalene.isDone():
+        return False
+    return scalene_profiler.Scalene.should_trace(s)
+
 def main():
     try:
         from scalene import scalene_profiler
