@@ -1,5 +1,6 @@
 from scalene.scalene_profiler import Scalene
 import sys
+import time
 from typing import Optional
 import threading
 
@@ -13,7 +14,7 @@ def replacement_thread_join(scalene: Scalene) -> None:
     ) -> None:
         """We replace threading.Thread.join with this method which always
         periodically yields."""
-        start_time = scalene.get_wallclock_time()
+        start_time = time.perf_counter()
         interval = sys.getswitchinterval()
         while self.is_alive():
             scalene.set_thread_sleeping(threading.get_ident())
@@ -21,7 +22,7 @@ def replacement_thread_join(scalene: Scalene) -> None:
             scalene.reset_thread_sleeping(threading.get_ident())
             # If a timeout was specified, check to see if it's expired.
             if timeout:
-                end_time = scalene.get_wallclock_time()
+                end_time = time.perf_counter()
                 if end_time - start_time >= timeout:
                     return None
         return None
