@@ -3,11 +3,13 @@ import argparse
 
 class ScaleneArguments(argparse.Namespace):
     def __init__(self) -> None:
+        super(ScaleneArguments, self).__init__()
         self.cpu_only = False
         self.cpu_percent_threshold = 1
         # mean seconds between interrupts for CPU sampling.
         self.cpu_sampling_rate = 0.01
         self.html = False
+        self.json = False
         self.malloc_threshold = 100
         self.outfile = None
         self.pid = 0
