@@ -1,6 +1,6 @@
+import contextlib
 import pynvml
 
-
 class ScaleneGPU:
     """A wrapper around the nvidia device driver library (nvidia-ml-py)."""
 
@@ -8,34 +8,33 @@ class ScaleneGPU:
         self.__ngpus = 0
         self.__has_gpu = False
         self.__handle = []
-        try:
+        with contextlib.suppress(Exception):
             pynvml.nvmlInit()
             self.__has_gpu = True
             self.__ngpus = pynvml.nvmlDeviceGetCount()
             for i in range(self.__ngpus):
                 self.__handle.append(pynvml.nvmlDeviceGetHandleByIndex(i))
-        except:
-            pass
 
     def has_gpu(self) -> bool:
         return self.__has_gpu
 
     def nvml_reinit(self) -> None:
         self.__handle = []
-        try:
+        with contextlib.suppress(Exception):
             pynvml.nvmlInit()
             self.__ngpus = pynvml.nvmlDeviceGetCount()
             for i in range(self.__ngpus):
                 self.__handle.append(pynvml.nvmlDeviceGetHandleByIndex(i))
-        except:
-            pass
 
     def load(self) -> float:
         if self.__has_gpu:
-            l = 0.0
+            total_load = 0.0
             for i in range(self.__ngpus):
-                l += pynvml.nvmlDeviceGetUtilizationRates(self.__handle[i]).gpu
-            return (l / self.__ngpus) / 100.0
+                with contextlib.suppress(Exception):
+                    total_load += pynvml.nvmlDeviceGetUtilizationRates(
+                        self.__handle[i]
+                    ).gpu
+            return (total_load / self.__ngpus) / 100.0
         return 0.0
 
     def memory_used(self) -> int:
