@@ -1,4 +1,6 @@
-try:
+import contextlib
+
+with contextlib.suppress(Exception):
     from IPython.core.magic import (
         Magics,
         magics_class,
@@ -15,6 +17,7 @@ try:
     class ScaleneMagics(Magics):  # type: ignore
         def run_code(self, args: ScaleneArguments, code: str) -> None:
             import IPython
+
             # Create a file to hold the supplied code.
             # We encode the cell number in the string for later recovery.
             # The length of the history buffer lets us find the most recent string (this one).
@@ -47,18 +50,16 @@ try:
                 sys.argv = ["scalene"]
                 sys.argv.extend(line.split(" "))
                 (args, left) = ScaleneParseArgs.parse_args()
-                self.run_code(args, (" ").join(left))  # type: ignore
+                self.run_code(args, " ".join(left))  # type: ignore
 
     def load_ipython_extension(ip: Any) -> None:
         ip.register_magics(ScaleneMagics)
-        try:
+        with contextlib.suppress(Exception):
             # For some reason, this isn't loading correctly on the web.
             with open("scalene-usage.txt", "r") as usage:
-                str = usage.read()
-            ScaleneMagics.scrun.__doc__ = str
-            ScaleneMagics.scalene.__doc__ = str
-        except Exception:
-            pass
+                usage_str = usage.read()
+            ScaleneMagics.scrun.__doc__ = usage_str
+            ScaleneMagics.scalene.__doc__ = usage_str
         import textwrap
 
         print(
@@ -68,7 +69,3 @@ try:
                 )
             )
         )
-
-
-except:
-    pass
