@@ -2,8 +2,9 @@ import shutil
 import sys
 import tempfile
 
-from collections import OrderedDict
+from collections import defaultdict, OrderedDict
 from operator import itemgetter
+from pathlib import Path
 from rich.console import Console
 from rich.markdown import Markdown
 from rich.syntax import Syntax
@@ -12,10 +13,11 @@ from rich.text import Text
 from rich import box
 
 from scalene import sparkline
+from scalene.scalene_json import ScaleneJSON
 from scalene.syntaxline import SyntaxLine
-from scalene.scalene_statistics import *
+from scalene.scalene_statistics import Filename, LineNumber, ScaleneStatistics
 
-from typing import Callable, Union
+from typing import Any, Callable, Dict, List, Union
 
 
 class ScaleneOutput:
@@ -42,6 +44,7 @@ class ScaleneOutput:
     # Profile output methods
     def output_profile_line(
         self,
+        json: ScaleneJSON,
         fname: Filename,
         line_no: LineNumber,
         line: SyntaxLine,
@@ -56,143 +59,66 @@ class ScaleneOutput:
         reduced_profile: bool = False,
     ) -> bool:
         """Print at most one line of the profile (true == printed one)."""
-        if not force_print and not profile_this_code(fname, line_no):
+        obj = json.output_profile_line(
+            fname=fname,
+            line_no=line_no,
+            stats=stats,
+            profile_this_code=profile_this_code,
+            force_print=force_print,
+        )
+        if not obj:
             return False
-        current_max = stats.max_footprint
-        # Prepare output values.
-        n_cpu_samples_c = stats.cpu_samples_c[fname][line_no]
-        # Correct for negative CPU sample counts. This can happen
-        # because of floating point inaccuracies, since we perform
-        # subtraction to compute it.
-        if n_cpu_samples_c < 0:
-            n_cpu_samples_c = 0
-        n_cpu_samples_python = stats.cpu_samples_python[fname][line_no]
-        n_gpu_samples = stats.gpu_samples[fname][line_no]
-
-        # Compute percentages of CPU time.
-        if stats.total_cpu_samples != 0:
-            n_cpu_percent_c = n_cpu_samples_c * 100 / stats.total_cpu_samples
-            n_cpu_percent_python = (
-                n_cpu_samples_python * 100 / stats.total_cpu_samples
-            )
-        else:
-            n_cpu_percent_c = 0
-            n_cpu_percent_python = 0
-
-        if stats.total_gpu_samples != 0:
-            n_gpu_percent = n_gpu_samples * 100 / stats.total_gpu_samples
-        else:
-            n_gpu_percent = 0
-
-        # Now, memory stats.
-        # Accumulate each one from every byte index.
-        n_malloc_mb = 0.0
-        n_python_malloc_mb = 0.0
-        n_free_mb = 0.0
-        for index in stats.bytei_map[fname][line_no]:
-            mallocs = stats.memory_malloc_samples[fname][line_no][index]
-            n_malloc_mb += mallocs
-            n_python_malloc_mb += stats.memory_python_samples[fname][line_no][
-                index
-            ]
-            frees = stats.memory_free_samples[fname][line_no][index]
-            n_free_mb += frees
-
-        n_usage_fraction = (
-            0
-            if not stats.total_memory_malloc_samples
-            else n_malloc_mb / stats.total_memory_malloc_samples
-        )
-        n_python_fraction = (
-            0
-            if not n_malloc_mb
-            else n_python_malloc_mb
-            / stats.total_memory_malloc_samples  # was / n_malloc_mb
-        )
-
-        if False:
-            # Currently disabled; possibly use in another column?
-            # Normalize by number of samples ("net *average*")
-            for bytei in stats.memory_malloc_count[fname][
-                line_no
-            ]:  # type : ignore
-                count = stats.memory_malloc_count[fname][line_no][bytei]
-                if count > 0:
-                    n_malloc_mb /= count
-                    n_python_malloc_mb /= count
-            for bytei in stats.memory_free_count[fname][line_no]:
-                count = stats.memory_free_count[fname][line_no][bytei]
-                if count > 0:
-                    n_free_mb /= count
-
-        n_growth_mb = n_malloc_mb - n_free_mb
-        if -1 < n_growth_mb < 1:
+        if -1 < obj["n_growth_mb"] < 1:
             # Don't print out "-0" or anything below 1.
-            n_growth_mb = 0
-
-        n_cpu_percent = n_cpu_percent_c + n_cpu_percent_python
-        n_sys_percent = n_cpu_percent * (
-            1.0 - (stats.cpu_utilization[fname][line_no].mean())
-        )
-
-        # Adjust CPU time by utilization.
-        n_cpu_percent_python *= stats.cpu_utilization[fname][line_no].mean()
-        n_cpu_percent_c *= stats.cpu_utilization[fname][line_no].mean()
+            obj["n_growth_mb"] = 0
 
         # Finally, print results.
         n_cpu_percent_c_str: str = (
-            "" if n_cpu_percent_c < 1 else f"{n_cpu_percent_c:5.0f}%"
+            ""
+            if obj["n_cpu_percent_c"] < 1
+            else f"{obj['n_cpu_percent_c']:5.0f}%"
         )
 
         n_gpu_percent_str: str = (
-            "" if n_gpu_percent < 1 else f"{n_gpu_percent:3.0f}%"
+            "" if obj["n_gpu_percent"] < 1 else f"{obj['n_gpu_percent']:3.0f}%"
         )
 
         n_cpu_percent_python_str: str = (
-            "" if n_cpu_percent_python < 1 else f"{n_cpu_percent_python:5.0f}%"
+            ""
+            if obj["n_cpu_percent_python"] < 1
+            else f"{obj['n_cpu_percent_python']:5.0f}%"
         )
         n_growth_mem_str = ""
-        if n_growth_mb < 1024:
+        if obj["n_growth_mb"] < 1024:
             n_growth_mem_str = (
                 ""
-                if (not n_growth_mb and not n_usage_fraction)
-                else f"{n_growth_mb:5.0f}M"
+                if (not obj["n_growth_mb"] and not obj["n_usage_fraction"])
+                else f"{obj['n_growth_mb']:5.0f}M"
             )
         else:
             n_growth_mem_str = (
                 ""
-                if (not n_growth_mb and not n_usage_fraction)
-                else f"{(n_growth_mb / 1024):5.2f}G"
+                if (not obj["n_growth_mb"] and not obj["n_usage_fraction"])
+                else f"{(obj['n_growth_mb'] / 1024):5.2f}G"
             )
 
         n_usage_fraction_str: str = (
             ""
-            if n_usage_fraction < 0.01
-            else f"{(100 * n_usage_fraction):4.0f}%"
+            if obj["n_usage_fraction"] < 0.01
+            else f"{(100 * obj['n_usage_fraction']):4.0f}%"
         )
         n_python_fraction_str: str = (
             ""
-            if n_python_fraction < 0.01
-            else f"{(n_python_fraction * 100):4.0f}%"
+            if obj["n_python_fraction"] < 0.01
+            else f"{(obj['n_python_fraction'] * 100):4.0f}%"
         )
-        n_copy_b = stats.memcpy_samples[fname][line_no]
-        if stats.elapsed_time:
-            n_copy_mb_s = n_copy_b / (1024 * 1024 * stats.elapsed_time)
-        else:
-            n_copy_mb_s = 0
         n_copy_mb_s_str: str = (
-            "" if n_copy_mb_s < 0.5 else f"{n_copy_mb_s:6.0f}"
+            "" if obj["n_copy_mb_s"] < 0.5 else f"{obj['n_copy_mb_s']:6.0f}"
         )
 
-        # Only report utilization where there is more than 1% CPU total usage,
-        # and the standard error of the mean is low (meaning it's an accurate estimate).
+        # Only report utilization where there is more than 1% CPU total usage.
         sys_str: str = (
-            ""
-            if n_sys_percent < 1
-            # or stats.cpu_utilization[fname][line_no].size() <= 1
-            # or stats.cpu_utilization[fname][line_no].sem() > 0.025
-            # or stats.cpu_utilization[fname][line_no].mean() > 0.99
-            else f"{n_sys_percent:4.0f}%"
+            "" if obj["n_sys_percent"] < 1 else f"{obj['n_sys_percent']:4.0f}%"
         )
         if not is_function_summary:
             print_line_no = "" if suppress_lineno_print else str(line_no)
@@ -205,12 +131,12 @@ class ScaleneOutput:
         if profile_memory:
             spark_str: str = ""
             # Scale the sparkline by the usage fraction.
-            samples = stats.per_line_footprint_samples[fname][line_no]
-            for i in range(0, len(samples.get())):
-                samples.get()[i] *= n_usage_fraction
-            if samples.get():
+            samples = obj["memory_samples"]
+            for i in range(0, len(samples)):
+                samples[i] *= obj["n_usage_fraction"]
+            if samples:
                 _, _, spark_str = sparkline.generate(
-                    samples.get()[0 : samples.len()], 0, current_max
+                    samples, 0, stats.max_footprint
                 )
 
             # Red highlight
@@ -220,8 +146,12 @@ class ScaleneOutput:
             ngpus: Any = ""
 
             if (
-                n_usage_fraction >= self.highlight_percentage
-                or (n_cpu_percent_c + n_cpu_percent_python + n_gpu_percent)
+                obj["n_usage_fraction"] >= self.highlight_percentage
+                or (
+                    obj["n_cpu_percent_c"]
+                    + obj["n_cpu_percent_python"]
+                    + obj["n_gpu_percent"]
+                )
                 >= self.highlight_percentage
             ):
                 ncpps = Text.assemble((n_cpu_percent_python_str, "bold red"))
@@ -270,7 +200,9 @@ class ScaleneOutput:
 
             # Red highlight
             if (
-                n_cpu_percent_c + n_cpu_percent_python + n_gpu_percent
+                obj["n_cpu_percent_c"]
+                + obj["n_cpu_percent_python"]
+                + obj["n_gpu_percent"]
             ) >= self.highlight_percentage:
                 ncpps = Text.assemble((n_cpu_percent_python_str, "bold red"))
                 ncpcs = Text.assemble((n_cpu_percent_c_str, "bold red"))
@@ -308,15 +240,16 @@ class ScaleneOutput:
         stats: ScaleneStatistics,
         pid: int,
         profile_this_code: Callable[[Filename, LineNumber], bool],
-        python_alias_dir: Filename,
+        python_alias_dir: Path,
         profile_memory: bool = True,
         reduced_profile: bool = False,
     ) -> bool:
         """Write the profile out."""
         # Get the children's stats, if any.
+        json = ScaleneJSON()
+        json.gpu = self.gpu
         if not pid:
             stats.merge_stats(python_alias_dir)
-        current_max: float = stats.max_footprint
         # If we've collected any samples, dump them.
         if (
             not stats.total_cpu_samples
@@ -344,7 +277,7 @@ class ScaleneOutput:
             if len(samples.get()) > 0:
                 # Output a sparkline as a summary of memory usage over time.
                 _, _, spark_str = sparkline.generate(
-                    samples.get()[0 : samples.len()], 0, current_max
+                    samples.get()[0 : samples.len()], 0, stats.max_footprint
                 )
                 # Compute growth rate (slope), between 0 and 1.
                 if stats.allocation_velocity[1] > 0:
@@ -354,12 +287,12 @@ class ScaleneOutput:
                         / stats.allocation_velocity[1]
                     )
                 # If memory used is > 1GB, use GB as the unit.
-                if current_max > 1024:
+                if stats.max_footprint > 1024:
                     mem_usage_line = Text.assemble(
                         "Memory usage: ",
                         ((spark_str, "dark_green")),
                         (
-                            f" (max: {(current_max / 1024):6.2f}GB, growth rate: {growth_rate:3.0f}%)\n"
+                            f" (max: {(stats.max_footprint / 1024):6.2f}GB, growth rate: {growth_rate:3.0f}%)\n"
                         ),
                     )
                 else:
@@ -368,7 +301,7 @@ class ScaleneOutput:
                         "Memory usage: ",
                         ((spark_str, "dark_green")),
                         (
-                            f" (max: {current_max:6.2f}MB, growth rate: {growth_rate:3.0f}%)\n"
+                            f" (max: {stats.max_footprint:6.2f}MB, growth rate: {growth_rate:3.0f}%)\n"
                         ),
                     )
 
@@ -459,39 +392,66 @@ class ScaleneOutput:
             )
 
             tbl.add_column(
-                Markdown("Line", style="dim"), style="dim", justify="right", no_wrap=True, width=4
+                Markdown("Line", style="dim"),
+                style="dim",
+                justify="right",
+                no_wrap=True,
+                width=4,
             )
             tbl.add_column(
-                Markdown("Time  " + "\n" + "_Python_", style="blue"), style="blue", no_wrap=True, width=6
+                Markdown("Time  " + "\n" + "_Python_", style="blue"),
+                style="blue",
+                no_wrap=True,
+                width=6,
             )
             tbl.add_column(
-                Markdown("––––––  \n_native_", style="blue"), style="blue", no_wrap=True, width=6
+                Markdown("––––––  \n_native_", style="blue"),
+                style="blue",
+                no_wrap=True,
+                width=6,
             )
             tbl.add_column(
-                Markdown("––––––  \n_system_", style="blue"), style="blue", no_wrap=True, width=6
+                Markdown("––––––  \n_system_", style="blue"),
+                style="blue",
+                no_wrap=True,
+                width=6,
             )
             if self.gpu:
                 tbl.add_column(
-                    Markdown("––––––  \n_GPU_", style="yellow4"), style="yellow4", no_wrap=True, width=6
+                    Markdown("––––––  \n_GPU_", style="yellow4"),
+                    style="yellow4",
+                    no_wrap=True,
+                    width=6,
                 )
 
             other_columns_width = 0  # Size taken up by all columns BUT code
 
             if profile_memory:
                 tbl.add_column(
-                    Markdown("Memory  \n_Python_", style="dark_green"), style="dark_green", no_wrap=True, width=7
+                    Markdown("Memory  \n_Python_", style="dark_green"),
+                    style="dark_green",
+                    no_wrap=True,
+                    width=7,
                 )
                 tbl.add_column(
-                    Markdown("––––––  \n_net_", style="dark_green"), style="dark_green", no_wrap=True, width=6
+                    Markdown("––––––  \n_avg_", style="dark_green"),
+                    style="dark_green",
+                    no_wrap=True,
+                    width=6,
                 )
                 tbl.add_column(
-                    Markdown("–––––––––––  \n_timeline_/%", style="dark_green"),
+                    Markdown(
+                        "–––––––––––  \n_timeline_/%", style="dark_green"
+                    ),
                     style="dark_green",
                     no_wrap=True,
                     width=14,
                 )
                 tbl.add_column(
-                    Markdown("Copy  \n_(MB/s)_", style="yellow4"), style="yellow4", no_wrap=True, width=6
+                    Markdown("Copy  \n_(MB/s)_", style="yellow4"),
+                    style="yellow4",
+                    no_wrap=True,
+                    width=6,
                 )
                 other_columns_width = 75 + (6 if self.gpu else 0)
                 tbl.add_column(
@@ -508,6 +468,11 @@ class ScaleneOutput:
                 )
 
             # Print out the the profile for the source, line by line.
+            if fname == "<BOGUS>":
+                continue
+            if not fname:
+                continue
+            # Print out the profile for the source, line by line.
             with open(fname, "r") as source_file:
                 # We track whether we should put in ellipsis (for reduced profiles)
                 # or not.
@@ -516,23 +481,13 @@ class ScaleneOutput:
                 # Generate syntax highlighted version for the whole file,
                 # which we will consume a line at a time.
                 # See https://github.com/willmcgugan/rich/discussions/965#discussioncomment-314233
-                syntax_highlighted = None
-                if self.html:
-                    syntax_highlighted = Syntax(
-                        code_lines,
-                        "python",
-                        theme="default",
-                        line_numbers=False,
-                        code_width=None,
-                    )
-                else:
-                    syntax_highlighted = Syntax(
-                        code_lines,
-                        "python",
-                        theme="vim",
-                        line_numbers=False,
-                        code_width=None,
-                    )
+                syntax_highlighted = Syntax(
+                    code_lines,
+                    "python",
+                    theme="default" if self.html else "vim",
+                    line_numbers=False,
+                    code_width=None,
+                )
                 capture_console = Console(
                     width=column_width - other_columns_width,
                     force_terminal=True,
@@ -546,13 +501,14 @@ class ScaleneOutput:
                 for line_no, line in enumerate(formatted_lines, start=1):
                     old_did_print = did_print
                     did_print = self.output_profile_line(
-                        fname,
-                        LineNumber(line_no),
-                        line,
-                        console,
-                        tbl,
-                        stats,
-                        profile_this_code,
+                        json=json,
+                        fname=fname,
+                        line_no=LineNumber(line_no),
+                        line=line,
+                        console=console,
+                        tbl=tbl,
+                        stats=stats,
+                        profile_this_code=profile_this_code,
                         profile_memory=profile_memory,
                         force_print=False,
                         suppress_lineno_print=False,
@@ -595,31 +551,23 @@ class ScaleneOutput:
                 ):
                     if fn_name == fname:
                         continue
-                    if self.html:
-                        syntax_highlighted = Syntax(
-                            fn_name,
-                            "python",
-                            theme="default",
-                            line_numbers=False,
-                            code_width=None,
-                        )
-                    else:
-                        syntax_highlighted = Syntax(
-                            fn_name,
-                            "python",
-                            theme="vim",
-                            line_numbers=False,
-                            code_width=None,
-                        )
+                    syntax_highlighted = Syntax(
+                        fn_name,
+                        "python",
+                        theme="default" if self.html else "vim",
+                        line_numbers=False,
+                        code_width=None,
+                    )
                     # force print, suppress line numbers
                     self.output_profile_line(
-                        fn_name,
-                        LineNumber(1),
-                        syntax_highlighted,  # type: ignore
-                        console,
-                        tbl,
-                        fn_stats,
-                        profile_this_code,
+                        json=json,
+                        fname=fn_name,
+                        line_no=LineNumber(1),
+                        line=syntax_highlighted,  # type: ignore
+                        console=console,
+                        tbl=tbl,
+                        stats=fn_stats,
+                        profile_this_code=profile_this_code,
                         profile_memory=profile_memory,
                         force_print=True,
                         suppress_lineno_print=True,
@@ -629,36 +577,53 @@ class ScaleneOutput:
 
             console.print(tbl)
 
-            # Report top K lines (currently 5) in terms of net memory consumption.
-            net_mallocs: Dict[LineNumber, float] = defaultdict(float)
+            # Report top K lines (currently 5) in terms of average memory consumption.
+            avg_mallocs: Dict[LineNumber, float] = defaultdict(float)
             for line_no in stats.bytei_map[fname]:
                 for bytecode_index in stats.bytei_map[fname][line_no]:
-                    net_mallocs[line_no] += (
+                    avg_mallocs[line_no] += (
                         stats.memory_malloc_samples[fname][line_no][
                             bytecode_index
                         ]
-                        - stats.memory_free_samples[fname][line_no][
-                            bytecode_index
-                        ]
+                        #- stats.memory_free_samples[fname][line_no][
+                        #    bytecode_index
+                        #]
                     )
-            net_mallocs = OrderedDict(
-                sorted(net_mallocs.items(), key=itemgetter(1), reverse=True)
+                    count = stats.memory_malloc_count[fname][line_no][bytecode_index]
+                    if count:
+                        avg_mallocs[line_no] /= count
+
+            avg_mallocs = OrderedDict(
+                sorted(avg_mallocs.items(), key=itemgetter(1), reverse=True)
             )
-            if len(net_mallocs) > 0:
-                console.print("Top net memory consumption, by line:")
+            # Print the top N lines by net memory consumption, as long
+            # as they are above some threshold MB in size.
+            print_top_mallocs_count = 5
+            print_top_mallocs_threshold_mb = 1
+            if len(avg_mallocs) > 0:
+                printed_header = False
                 number = 1
-                for net_malloc_lineno in net_mallocs:
-                    if net_mallocs[net_malloc_lineno] <= 1:
+                for avg_malloc_lineno in avg_mallocs:
+                    # Don't print lines with less than the threshold MB allocated.
+                    if (
+                        avg_mallocs[avg_malloc_lineno]
+                        <= print_top_mallocs_threshold_mb
+                    ):
                         break
-                    if number > 5:
+                    # Only print the top N.
+                    if number > print_top_mallocs_count:
                         break
+                    # Print the header only if we are printing something (and only once).
+                    if not printed_header:
+                        console.print("Top average memory consumption, by line:")
+                        printed_header = True
                     output_str = (
                         "("
                         + str(number)
                         + ") "
-                        + ("%5.0f" % (net_malloc_lineno))
+                        + ("%5.0f" % (avg_malloc_lineno))
                         + ": "
-                        + ("%5.0f" % (net_mallocs[net_malloc_lineno]))
+                        + ("%5.0f" % (avg_mallocs[avg_malloc_lineno]))
                         + " MB"
                     )
                     console.print(Markdown(output_str, style="dark_green"))
@@ -671,7 +636,6 @@ class ScaleneOutput:
             leak_reporting_threshold = 0.05
             leaks = []
             if growth_rate / 100 > growth_rate_threshold:
-                vec = list(stats.leak_score[fname].values())
                 keys = list(stats.leak_score[fname].keys())
                 for index, item in enumerate(stats.leak_score[fname].values()):
                     # See https://en.wikipedia.org/wiki/Rule_of_succession
@@ -683,7 +647,7 @@ class ScaleneOutput:
                             (
                                 keys[index],
                                 1 - expected_leak,
-                                net_mallocs[keys[index]],
+                                avg_mallocs[keys[index]],
                             )
                         )
                 if len(leaks) > 0:
