@@ -2,25 +2,30 @@ from scalene.scalene_arguments import ScaleneArguments
 from scalene.scalene_version import scalene_version
 
 from typing import (
+    Any,
     List,
+    NoReturn,
+    Optional,
     Tuple,
 )
 from textwrap import dedent
 import argparse
+import contextlib
 import sys
 
 
 class RichArgParser(argparse.ArgumentParser):
-
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any):
         from rich.console import Console
+
         self.console = Console()
         super().__init__(*args, **kwargs)
-        
-    def _print_message(self, message, file=None):
+
+    def _print_message(self, message: Optional[str], file: Any = None) -> None:
         if message:
             self.console.print(message)
-                
+
+
 class StopJupyterExecution(Exception):
     """NOP exception to enable clean exits from within Jupyter notebooks."""
 
@@ -30,21 +35,19 @@ class StopJupyterExecution(Exception):
 
 class ScaleneParseArgs:
     @staticmethod
-    def clean_exit(code: int) -> None:
+    def clean_exit(code: object = 0) -> NoReturn:
         """Replacement for sys.exit that exits cleanly from within Jupyter notebooks."""
         raise StopJupyterExecution
 
     @staticmethod
     def parse_args() -> Tuple[argparse.Namespace, List[str]]:
         # In IPython, intercept exit cleanly (because sys.exit triggers a backtrace).
-        try:
+        with contextlib.suppress(BaseException):
             from IPython import get_ipython
 
             if get_ipython():
-                sys.exit = clean_exit  # type: ignore
-                sys._exit = clean_exit
-        except:
-            pass
+                sys.exit = ScaleneParseArgs.clean_exit
+                sys._exit = ScaleneParseArgs.clean_exit  # type: ignore
         defaults = ScaleneArguments()
         usage = dedent(
             f"""[b]Scalene[/b]: a high-precision CPU and memory profiler, version {scalene_version}
@@ -76,7 +79,7 @@ for the process ID that Scalene reports. For example:
 """
         )
 
-        parser = RichArgParser( # argparse.ArgumentParser(
+        parser = RichArgParser(  # argparse.ArgumentParser(
             prog="scalene",
             description=usage,
             epilog=epilog,
@@ -108,6 +111,16 @@ for the process ID that Scalene reports. For example:
             + str("html" if defaults.html else "text")
             + "[/blue])",
         )
+        parser.add_argument(
+            "--json",
+            dest="json",
+            action="store_const",
+            const=True,
+            default=defaults.json,
+            help="output as JSON (default: [blue]"
+            + str("json" if defaults.json else "text")
+            + "[/blue])",
+        )
         parser.add_argument(
             "--reduced-profile",
             dest="reduced_profile",
@@ -129,7 +142,11 @@ for the process ID that Scalene reports. For example:
             const=True,
             default=defaults.cpu_only,
             help="only profile CPU+GPU time (default: [blue]profile "
-            + ("CPU only" if defaults.cpu_only else "CPU+GPU, memory, and copying")
+            + (
+                "CPU only"
+                if defaults.cpu_only
+                else "CPU+GPU, memory, and copying"
+            )
             + "[/blue])",
         )
         parser.add_argument(
@@ -151,7 +168,7 @@ for the process ID that Scalene reports. For example:
             dest="profile_only",
             type=str,
             default=defaults.profile_only,
-            help="profile only code in files matching the given strings, separated by commas (default: [blue]"
+            help="profile only code in filenames that contain the given strings, separated by commas (default: [blue]"
             + (
                 "no restrictions"
                 if not defaults.profile_only
@@ -222,7 +239,10 @@ for the process ID that Scalene reports. For example:
         args, left = parser.parse_known_args()
         left += args.unused_args
         import re
-        in_jupyter_notebook = len(sys.argv) >= 1 and re.match("<ipython-input-([0-9]+)-.*>", sys.argv[0])
+
+        in_jupyter_notebook = len(sys.argv) >= 1 and re.match(
+            "<ipython-input-([0-9]+)-.*>", sys.argv[0]
+        )
         # If the user did not enter any commands (just `scalene` or `python3 -m scalene`),
         # print the usage information and bail.
         if not in_jupyter_notebook and (len(sys.argv) + len(left) == 1):
