@@ -1,4 +1,5 @@
 import argparse
+import contextlib
 import os
 import platform
 import scalene
@@ -9,6 +10,7 @@ import sys
 
 from typing import Dict
 
+
 class ScalenePreload:
     @staticmethod
     def get_preload_environ(args: argparse.Namespace) -> Dict[str, str]:
@@ -16,13 +18,21 @@ class ScalenePreload:
 
         if sys.platform == "linux":
             if not args.cpu_only:
-                env["LD_PRELOAD"] = os.path.join(scalene.__path__[0], "libscalene.so")
-                env["PYTHONMALLOC"] = "malloc"
+                env["LD_PRELOAD"] = os.path.join(
+                    scalene.__path__[0], "libscalene.so"  # type: ignore
+                )
+                # Disable command-line specified PYTHONMALLOC.
+                if "PYTHONMALLOC" in env:
+                    del env["PYTHONMALLOC"]
 
         elif sys.platform == "darwin":
             if not args.cpu_only:
-                env["DYLD_INSERT_LIBRARIES"] = os.path.join(scalene.__path__[0], "libscalene.dylib")
-                env["PYTHONMALLOC"] = "malloc"
+                env["DYLD_INSERT_LIBRARIES"] = os.path.join(
+                    scalene.__path__[0], "libscalene.dylib"
+                )
+                # Disable command-line specified PYTHONMALLOC.
+                if "PYTHONMALLOC" in env:
+                    del env["PYTHONMALLOC"]
             # required for multiprocessing support, even without libscalene
             env["OBJC_DISABLE_INITIALIZE_FORK_SAFETY"] = "YES"
 
@@ -56,14 +66,12 @@ class ScalenePreload:
                 "Scalene warning: currently only 64-bit x86-64 and ARM platforms are supported for memory and copy profiling."
             )
 
-        try:
+        with contextlib.suppress(Exception):
             from IPython import get_ipython
 
             if get_ipython():
                 sys.exit = Scalene.clean_exit  # type: ignore
-                sys._exit = Scalene.clean_exit
-        except:
-            pass
+                sys._exit = Scalene.clean_exit  # type: ignore
 
         # Start a subprocess with the required environment variables,
         # which may include preloading libscalene
@@ -76,10 +84,8 @@ class ScalenePreload:
                 "-m",
                 "scalene",
             ] + sys.argv[1:]
-            result = subprocess.Popen(
-                new_args, close_fds=True, shell=False
-            )
-            try:
+            result = subprocess.Popen(new_args, close_fds=True, shell=False)
+            with contextlib.suppress(Exception):
                 # If running in the background, print the PID.
                 if os.getpgrp() != os.tcgetpgrp(sys.stdout.fileno()):
                     # In the background.
@@ -90,9 +96,11 @@ class ScalenePreload:
                     print(
                         f"  to resume profiling:  python3 -m scalene.profile --on  --pid {result.pid}"
                     )
-            except:
+            try:
+                result.wait()
+            except KeyboardInterrupt:
+                result.returncode = 0
                 pass
-            result.wait()
             if result.returncode < 0:
                 print(
                     "Scalene error: received signal",
