@@ -15,6 +15,7 @@
 
 #include "printf.h"
 #include "sampler.hpp"
+#include "pywhere.hpp"
 
 #if !defined(__APPLE__) && !defined(_WIN32)
 #include <endian.h>
@@ -165,9 +166,6 @@ void *memcpy_musl(void *dest, const void *src, size_t n) {
   return dest;
 }
 
-#if defined(__x86_64__)
-#include "rtememcpy.h"
-#endif
 #include "samplefile.hpp"
 
 template <uint64_t MemcpySamplingRateBytes>
@@ -180,9 +178,8 @@ class MemcpySampler {
 
  public:
   MemcpySampler()
-      : _samplefile((char *)"/tmp/scalene-memcpy-signal%d",
-                    (char *)"/tmp/scalene-memcpy-lock%d",
-                    (char *)"/tmp/scalene-memcpy-init%d"),
+      : _samplefile("/tmp/scalene-memcpy-signal%d",
+                    "/tmp/scalene-memcpy-lock%d", "/tmp/scalene-memcpy-init%d"),
         _interval(MemcpySamplingRateBytes),
         _memcpyOps(0),
         _memcpyTriggered(0) {
@@ -239,7 +236,6 @@ class MemcpySampler {
     return ::memcpy(dst, src, n);
 #else
     return memcpy_musl(dst, src, n);
-    //    return rte_memcpy(dst, src, n);
 #endif
   }
 
@@ -266,21 +262,23 @@ class MemcpySampler {
   }
 
   void incrementMemoryOps(int n) {
-    _memcpyOps += n;
-    auto sampleMemop = _memcpySampler.sample(n);
-    if (unlikely(sampleMemop)) {
-      writeCount();
-      _memcpyTriggered++;
-      _memcpyOps = 0;
+    if (pythonDetected()) {
+      _memcpyOps += n;
+      auto sampleMemop = _memcpySampler.sample(n);
+      if (unlikely(sampleMemop)) {
+        writeCount();
+        _memcpyTriggered++;
+        _memcpyOps = 0;
 #if !SCALENE_DISABLE_SIGNALS
-      raise(MemcpySignal);
+        raise(MemcpySignal);
 #endif
+      }
     }
   }
 
+  uint64_t _interval;
   uint64_t _memcpyOps;
   unsigned long long _memcpyTriggered;
-  uint64_t _interval;
   static constexpr int FILENAME_LENGTH = 255;
   char scalene_memcpy_signal_filename[FILENAME_LENGTH];
 
@@ -288,7 +286,7 @@ class MemcpySampler {
     char buf[FILENAME_LENGTH];
     snprintf(buf, FILENAME_LENGTH, "%d,%d,%d\n\n", _memcpyTriggered, _memcpyOps,
              getpid());
-    _samplefile.writeToFile(buf, 0);
+    _samplefile.writeToFile(buf);
   }
 };
 
